"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TaskEither = require("fp-ts/lib/TaskEither");
const pipeable_1 = require("fp-ts/lib/pipeable");
const R = require("ramda");
const types_1 = require("./types");
function describeStatement(pgClient, sql, paramNames) {
    return pipeable_1.pipe(TaskEither.tryCatch(() => pgClient.query({ text: sql, describe: true }), error => pgErrorToString(error, sql)), TaskEither.map(queryResult => describeResult(sql, paramNames, queryResult)));
}
exports.describeStatement = describeStatement;
function describeResult(sql, paramNames, queryResult) {
    return {
        columns: queryResult.fields.map(field => ({
            name: field.name,
            nullable: true,
            type: types_1.ValueType.any(field.dataTypeID),
        })),
        rowCount: 'many',
        params: R.zipWith((name, type) => ({
            name,
            nullable: false,
            type: types_1.ValueType.any(type),
        }), paramNames, queryResult.params),
        sql,
    };
}
function pgErrorToString(error, sql) {
    const sourceLines = sql.split('\n');
    const errorPos = error.position && findPositionInFile(Number(error.position), sourceLines);
    if (errorPos) {
        return formatError(error, errorPos, sourceLines);
    }
    return error.message || '';
}
exports.pgErrorToString = pgErrorToString;
function findPositionInFile(characterOffset, sourceLines) {
    const init = { offset: 0, lineNo: 1, result: null };
    const final = sourceLines.reduce(({ offset, lineNo, result }, line) => ({
        lineNo: lineNo + 1,
        offset: offset + line.length + 1,
        result: offset <= characterOffset && characterOffset < offset + line.length + 1
            ? { line: lineNo, column: characterOffset - offset }
            : result,
    }), init);
    return final.result;
}
function formatError(error, errorPos, sourceLines) {
    const result = [];
    result.push(`${error.severity}:  ${error.message}`);
    result.push('');
    let line;
    for (line = Math.max(errorPos.line - 19, 1); line <= errorPos.line; line++) {
        result.push(`${leftPad(String(line), 5)}| ${sourceLines[line - 1]}`);
    }
    result.push(errorMarker(errorPos.column + 7));
    if (error.hint) {
        result.push(`HINT:  ${error.hint}`);
    }
    return result.join('\n');
}
function errorMarker(column) {
    const arr = [];
    for (let i = 0; i < column - 1; i++) {
        arr.push(' ');
    }
    arr.push('^');
    return arr.join('');
}
function leftPad(s, width) {
    while (s.length < width) {
        s = ' ' + s;
    }
    return s;
}
